#---
# Excerpted from "Agile Web Development with Rails"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com for more book information.
#---
class StoreController < ApplicationController
  
  def index
    @products = Product.salable_items
  end

  def add_to_cart
    product = Product.find(params[:id])
    @cart = find_cart
    @cart.add_product(product)
    redirect_to(:action => 'display_cart')
  rescue
    logger.error("Attempt to access invalid product #{params[:id]}")
    redirect_to_index('Invalid product')
  end
  
  def display_cart
    @cart = find_cart
    @items = @cart.items
    if @items.empty?
      redirect_to_index("Your cart is currently empty")
    end
  end

  def empty_cart
    @cart = find_cart
    @cart.empty!
    redirect_to_index('Your cart is now empty')
  end

  
  def checkout
    @cart = find_cart
    @items = @cart.items
    if @items.empty?
      redirect_to_index("There's nothing in your cart!")
    else
      @order = Order.new
    end
  end
  

  # called from checkout view
  
  def save_order
    @cart = find_cart
    @order = Order.new(params[:order])   
    @order.line_items << @cart.items       
    if @order.save                        
      @cart.empty!
      redirect_to_index('Thank you for your order.')
    else
      render(:action => 'checkout')       
    end
  end
  

  private

  # Return a cart object. If we already have one cached
  # in the session, use it, otherwise create a new one 
  # and add it to the session
  def find_cart
    session[:cart] ||= Cart.new
  end

  # Set the notice if a parameter is givem, then redirect back
  # the the product listing

  def redirect_to_index(msg = nil)
    flash[:notice] = msg if msg
    redirect_to(:action => 'index')
  end

end
