#---
# Excerpted from "Agile Web Development with Rails"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com for more book information.
#---
require "rubygems"
require_gem "actionpack"
require 'action_controller/url_rewriter'

require 'fake_environment'



ActionController::Base.debug_routes = true


ActionController::Routing::Routes.draw do |map|

  # Straight 'http://my.app/blog/' displays the index
  map.index   "blog/", 
              :controller   => "blog",
              :action       => "index"

  # Return articles for a year, year/month, or year/month/day
  map.date "blog/:year/:month/:day", 
              :controller   => "blog",
              :action       => "show_date",
              :requirements => { :year  => /(19|20)\d\d/, 
                                 :month => /[01]?\d/, 
                                 :day   => /[0-3]?\d/},
              :day          => nil,
              :month        => nil

  # Show an article identified by an id
  map.show "blog/show/:id",
              :controller   => "blog",
              :action       => "show",
              :id           => /\d+/
  # Regular Rails routing for admin stuff
  map.admin "blog/:controller/:action/:id"

  # Catch-all so we can gracefully handle badly-formed requests
  map.catchall "*anything",
               :controller   => "blog",
               :action       => "unknown_request"

end

["blog",
 "blog/show/123",
 "blog/2004",
 "blog/2004/12",
 "blog/2004/12/25",
 "blog/article/edit/123",
 "blog/article/show_stats",
 "blog/wibble",
 "junk"].each do |url|
  
  puts "URL> #{url}"
  params = ActionController::Routing::Routes.recognize_path(url.split('/'))
  puts "@params = " + (params.map do |key, value|
    if value.kind_of?(FakeController)
      "#{key} => #{value.name}"
    else
      "#{key} => #{value}"
    end
  end).join(", ")
end

url = ActionController::UrlRewriter.new(Request.new("http://pragprog.com/blog"), {})

puts url.rewrite(:controller => "blog", "action" => "edit", :id => 1)


class ActionController::Base

  def dump_routes
    ActionController::Routing::NamedRoutes.install
    p index_url
  end
end

b = ActionController::Base.new
b.dump_routes

